/* 
 * Copyright 2012 ClinRisk Ltd.
 * 
 * This file is part of QRISK2-2012 (http://qrisk.org, http://svn.clinrisk.co.uk/qrisk2).
 * 
 * QRISK2-2012 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * QRISK2-2012 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with QRISK2-2012.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Additional terms
 * 
 * The following disclaimer must be displayed alongside any risk score generated by this code.
 *   The initial version of this file, to be found at http://svn.clinrisk.co.uk/qrisk2, faithfully implements QRISK2-2012.
 *   We have released this code under the GNU Lesser General Public License to enable others to implement the algorithm faithfully.
 *   However, the nature of the GNU Lesser General Public License is such that we cannot prevent, for example, someone altering the coefficients.
 *   We stress, therefore, that it is the responsibility of the end user to check that the source that they receive produces the same results as the original code posted at http://svn.clinrisk.co.uk/qrisk2.
 *   Inaccurate implementations of risk scores can lead to wrong patients being given the wrong treatment.
 * 
 * This file has been auto-generated.
 * XML source: Q68_qrisk2_2012_1_1.xml
 * STATA dta time stamp: 2 Jan 2012 23:10
 * This file was created on: Tue  3 Jan 2012 09:15:13 GMT
 */

#include <math.h>
#include <string.h>
#include <clinrisk/Q68_qrisk2_2012_1_1.h>
#include <clinrisk/utils.h>

static double cvd_male_raw(
int age,int b_AF,int b_ra,int b_renal,int b_treatedhyp,int b_type2,double bmi,int ethrisk,int fh_cvd,double rati,double sbp,int smoke_cat,int surv,double town
)
{
	double survivor[16] = {
		0,
		0.996994316577911,
		0.993941843509674,
		0.990756928920746,
		0.987367391586304,
		0.983686089515686,
		0.979941785335541,
		0.975925624370575,
		0.971652328968048,
		0.967043697834015,
		0.962060630321503,
		0.956596851348877,
		0.951202630996704,
		0.945618212223053,
		0.938979923725128,
		0.932796955108643
	};

	/* The conditional arrays */

	double Iethrisk[10] = {
		0,
		0,
		0.3163117410944710200000000,
		0.6091879104882792500000000,
		0.5958436006239369300000000,
		0.1142285841316822300000000,
		-0.3489215674318830200000000,
		-0.3603571055094617600000000,
		-0.2666173260881414300000000,
		-0.1208018847857646500000000
	};
	double Ismoke[5] = {
		0,
		0.2684290047224652300000000,
		0.5004920802983664500000000,
		0.6374758655070897900000000,
		0.7423631397556198500000000
	};

	/* Applying the fractional polynomial transforms */
	/* (which includes scaling)                      */

	double dage = age;
	dage=dage/10;
	double age_1 = pow(dage,-2);
	double age_2 = pow(dage,-2)*log(dage);
	double dbmi = bmi;
	dbmi=dbmi/10;
	double bmi_1 = log(dbmi);

	/* Centring the continuous variables */

	age_1 = age_1 - 0.044809196144342;
	age_2 = age_2 - 0.069573938846588;
	bmi_1 = bmi_1 - 0.970408320426941;
	rati = rati - 4.462830543518066;
	sbp = sbp - 133.357818603515620;
	town = town - -0.356812149286270;

	/* Start of Sum */
	double a=0;

	/* The conditional sums */

	a += Iethrisk[ethrisk];
	a += Ismoke[smoke_cat];

	/* Sum from continuous values */

	a += age_1 * 47.3163921673270660000000000;
	a += age_2 * -101.2362163621992300000000000;
	a += bmi_1 * 0.5424702715527407500000000;
	a += rati * 0.1442545616564490900000000;
	a += sbp * 0.0080703627452139003000000;
	a += town * 0.0364817567442078170000000;

	/* Sum from boolean values */

	a += b_AF * 0.7546727404172400800000000;
	a += b_ra * 0.3089197610059550800000000;
	a += b_renal * 0.7441080499101914200000000;
	a += b_treatedhyp * 0.4978222402431560800000000;
	a += b_type2 * 0.7776278619904274400000000;
	a += fh_cvd * 0.6965155800196054500000000;

	/* Sum from interaction terms */

	a += age_1 * (smoke_cat==1) * -3.8805330901643544000000000;
	a += age_1 * (smoke_cat==2) * -16.7031934095781480000000000;
	a += age_1 * (smoke_cat==3) * -15.3737691868177200000000000;
	a += age_1 * (smoke_cat==4) * -17.6452952709561190000000000;
	a += age_1 * b_AF * -7.0282038602850179000000000;
	a += age_1 * b_renal * -17.0146402956784510000000000;
	a += age_1 * b_treatedhyp * 33.9625265583895200000000000;
	a += age_1 * b_type2 * 12.7885598573212390000000000;
	a += age_1 * bmi_1 * 3.2680340443282705000000000;
	a += age_1 * fh_cvd * -17.9219200079209140000000000;
	a += age_1 * sbp * -0.1511050142259400800000000;
	a += age_1 * town * -2.5502442789058666000000000;
	a += age_2 * (smoke_cat==1) * 7.9708926855593845000000000;
	a += age_2 * (smoke_cat==2) * 23.6859391977921480000000000;
	a += age_2 * (smoke_cat==3) * 23.1370772208804420000000000;
	a += age_2 * (smoke_cat==4) * 26.8673734647379380000000000;
	a += age_2 * b_AF * 14.4518354362729100000000000;
	a += age_2 * b_renal * 28.2702331053646850000000000;
	a += age_2 * b_treatedhyp * -18.8166910013564140000000000;
	a += age_2 * b_type2 * 0.9629882124640055700000000;
	a += age_2 * bmi_1 * 10.5513753399433020000000000;
	a += age_2 * fh_cvd * 26.6046853897055510000000000;
	a += age_2 * sbp * 0.2910870792157880200000000;
	a += age_2 * town * 3.0069917093360048000000000;

	/* Calculate the score itself */
	double score = 100.0 * (1 - pow(survivor[surv], exp(a)) );
	return score;
}

static int cvd_male_validation(
int age,int b_AF,int b_ra,int b_renal,int b_treatedhyp,int b_type2,double bmi,int ethrisk,int fh_cvd,double rati,double sbp,int smoke_cat,int surv,double town,char *errorBuf,int errorBufSize
)
{
	int ok=1;
	*errorBuf=0;
	if (!i_in_range(age,30,84)) {
		ok=0;
		strlcat(errorBuf,"error: age must be in range (30,84)\n",errorBufSize);
	}
	if (!is_boolean(b_AF)) {
		ok=0;
		strlcat(errorBuf,"error: b_AF must be in range (0,1)\n",errorBufSize);
	}
	if (!is_boolean(b_ra)) {
		ok=0;
		strlcat(errorBuf,"error: b_ra must be in range (0,1)\n",errorBufSize);
	}
	if (!is_boolean(b_renal)) {
		ok=0;
		strlcat(errorBuf,"error: b_renal must be in range (0,1)\n",errorBufSize);
	}
	if (!is_boolean(b_treatedhyp)) {
		ok=0;
		strlcat(errorBuf,"error: b_treatedhyp must be in range (0,1)\n",errorBufSize);
	}
	if (!is_boolean(b_type2)) {
		ok=0;
		strlcat(errorBuf,"error: b_type2 must be in range (0,1)\n",errorBufSize);
	}
	if (!d_in_range(bmi,20,40)) {
		ok=0;
		strlcat(errorBuf,"error: bmi must be in range (20,40)\n",errorBufSize);
	}
	if (!i_in_range(ethrisk,1,9)) {
		ok=0;
		strlcat(errorBuf,"error: ethrisk must be in range (1,9)\n",errorBufSize);
	}
	if (!is_boolean(fh_cvd)) {
		ok=0;
		strlcat(errorBuf,"error: fh_cvd must be in range (0,1)\n",errorBufSize);
	}
	if (!d_in_range(rati,1,12)) {
		ok=0;
		strlcat(errorBuf,"error: rati must be in range (1,12)\n",errorBufSize);
	}
	if (!d_in_range(sbp,70,210)) {
		ok=0;
		strlcat(errorBuf,"error: sbp must be in range (70,210)\n",errorBufSize);
	}
	if (!i_in_range(smoke_cat,0,4)) {
		ok=0;
		strlcat(errorBuf,"error: smoke_cat must be in range (0,4)\n",errorBufSize);
	}
	if (!i_in_range(surv,1,10)) {
		ok=0;
		strlcat(errorBuf,"error: surv must be in range (1,10)\n",errorBufSize);
	}
	if (!d_in_range(town,-7,11)) {
		ok=0;
		strlcat(errorBuf,"error: town must be in range (-7,11)\n",errorBufSize);
	}
	return ok;
}

double cvd_male(
int age,int b_AF,int b_ra,int b_renal,int b_treatedhyp,int b_type2,double bmi,int ethrisk,int fh_cvd,double rati,double sbp,int smoke_cat,int surv,double town,int *error,char *errorBuf,int errorBufSize
)
{
	*error = 0;	int ok = cvd_male_validation(age,b_AF,b_ra,b_renal,b_treatedhyp,b_type2,bmi,ethrisk,fh_cvd,rati,sbp,smoke_cat,surv,town,errorBuf,errorBufSize);
	if(!ok) { 
		*error = 1;
		return 0.0;
	}
	return cvd_male_raw(age,b_AF,b_ra,b_renal,b_treatedhyp,b_type2,bmi,ethrisk,fh_cvd,rati,sbp,smoke_cat,surv,town);
}
